@@ -1,5 +1,5 @@
 # Locale::Po4a::Pod -- Convert POD data to PO file, for translation.
-# $Id: Chooser.pm,v 1.32 2006/06/12 18:33:30 nekral-guest Exp $
+# $Id: Chooser.pm,v 1.33 2006/08/26 14:19:56 nekral-guest Exp $
 #
 # This program is free software; you may redistribute it and/or modify it
 # under the terms of GPL (see COPYING).
@@ -38,6 +38,8 @@ sub new {
         $modname = 'Texinfo';
     } elsif ($module eq 'text') {
         $modname = 'Text';
+    } elsif ($module eq 'ini') {
+        $modname = 'Ini';
     } else {
         $modname = ucfirst($module);
     }
