@@ -145,16 +145,37 @@ should be preferred.
 
 =back
 
-=item B<untranslated>
+=item B<generated>
 
-=item B<noarg>
+This option specifies that the file was generated, and that po4a should not
+try to detect if the man pages was generated from another format.
+This permits to use po4a on generated man pages.
+This option does not take any argument.
 
-=item B<translate_joined>
+=item B<mdoc>
 
-=item B<translate_each>
+This option is only useful for mdoc pages.
 
-These options permit to specify the behavior of a new macro (defined with
-a .de request), or of a macro not supported by po4a.
+It selects a stricter support of the mdoc format by telling po4a not to
+translate the 'NAME' section.
+mdoc pages whose 'NAME' section is translated won't generate any header of
+footer.
+
+According to the groff_mdoc page, the NAME, SYNOPSIS and DESCRIPTION
+sections are mandatory.
+There are no known issues with translated SYNOPSIS or DESCRIPTION section,
+but you can also specify these sections this way:
+ -o mdoc=NAME,SYNOPSIS,DESCRIPTION
+
+This mdoc issue can also be solved with an addendum like this one:
+ PO4A-HEADER:mode=before;position=^.Dd
+ .TH DOCUMENT_TITLE 1 "Month day, year" OS "Section Name"
+
+
+=back
+
+The following options permit to specify the behavior of a new macro
+(defined with a .de request), or of a macro not supported by po4a.
 They take in argument a coma separated list of macros.
 For example:
 
@@ -163,15 +184,25 @@ For example:
 Note: if a macro is not supported by po4a and if you consider that it is a
 standard roff macro, you should submit it to the po4a development team.
 
+=over
+
+=item B<untranslated>
+
 B<untranslated> indicates that this macro (at its arguments) don't have to
 be translated.
 
+=item B<noarg>
+
 B<noarg> is like B<untranslated>, except that po4a will verify that no
 argument is added to this macro.
 
+=item B<translate_joined>
+
 B<translate_joined> indicates that po4a must propose to translate the
 arguments of the macro.
 
+=item B<translate_each>
+
 With B<translate_each>, the arguments will also be proposed for the
 translation, except that each one will be translated separately.
 
@@ -188,19 +219,12 @@ If you have a I<begin> (respectively I<end>) macro that has no I<end>
 I<begin> (like nf) as a counterpart.
 These macros (and their arguments) wont be translated.
 
-=item inline
+=item B<inline>
 
-This option permits to specify a list of coma-separated macros that must
+This option specifies a list of coma-separated macros that must
 not split the current paragraph. The string to translate will then contain
-I<foo E<lt>.bar baz quxE<gt> quux>, where I<foo> is the command that
-should be inlined.
-
-=item generated
-
-This option specifies that the file was generated, and that po4a should not
-try to detect if the man pages was generated from another format.
-This permits to use po4a on generated man pages.
-This option does not take any argument.
+I<foo E<lt>.bar baz quxE<gt> quux>, where I<bar> is the command that
+should be inlined, and I<baz qux> its arguments.
 
 =back
 
@@ -286,7 +310,7 @@ Some tests are regularly run on Linux boxes:
 =item *
 
 one third of the pages are refused because they were generated from
-another format suported by po4a (e.g. pod or SGML).
+another format supported by po4a (e.g. pod or SGML).
 
 =item *
 
@@ -392,6 +416,13 @@ my %inline = (
 # This variable indicates whether po4a should try to detect the generated
 # files.
 my $allow_generated = 0;
+# This hash indicates section name that should not be translated in mdoc
+# mode.
+# The groff's mdoc processor requires the NAME section, otherwise headers
+# and footers of the pages are not generated.
+# The mdoc_groff man page indicates that NAME, SYNOPSIS and DESCRIPTION
+# are mandatory.
+my %mdoc = ();
 sub initialize {
     my $self = shift;
     my %options = @_;
@@ -406,6 +437,7 @@ sub initialize {
     $self->{options}{'no_wrap'}='';
     $self->{options}{'inline'}='';
     $self->{options}{'generated'}='';
+    $self->{options}{'mdoc'}='';
 
     # reset the debug options
     %debug = ();
@@ -471,6 +503,15 @@ sub initialize {
     if (defined $options{'generated'}) {
         $allow_generated = 1;
     }
+    if (defined $options{'mdoc'}) {
+        if ($options{'mdoc'} eq 1) {
+            $mdoc{"NAME"} = 1;
+        } else {
+            foreach (split(/,/, $options{'mdoc'})) {
+                $mdoc{$_} = 1;
+            }
+        }
+    }
 }
 
 my @comments = ();
@@ -554,7 +595,7 @@ NEW_LINE:
         }
 
         if ($line =~ m/^[.']*$/) {
-            if ($c !~ m/^\s*$/) {
+            if ($c !~ m/^\s+$/) {
                 # This commentted line may be comment for the next paragraph
                 push @next_comments, $c;
             }
@@ -608,12 +649,14 @@ NEW_LINE:
                 $line = $macro."$insert_font\\f$1".$arg;
                 $ref = $r2;
             } elsif ($l2 =~ /^[.']/) {
-                die wrap_ref_mod($ref, "po4a::man", dgettext("po4a",
-                        "po4a does not support font modifiers followed by a ".
-                        "command.  You should either remove the font modifier ".
-                        "'%s', or integrate a \\f font modifier in the ".
-                        "following command ('%s')"
-                        ), $line, $l2);
+                warn wrap_ref_mod($ref, "po4a::man", dgettext("po4a",
+                         "Font modifiers followed by a command may disturb ".
+                         "po4a.  You should either remove the font modifier ".
+                         "'%s', or integrate a \\f font modifier in the ".
+                         "following command ('%s'), but continuing anyway."
+                         ), $line, $l2);
+                $line = "PO4A-INLINE:$line:PO4A-INLINE";
+                $self->SUPER::unshiftline($l2,$r2);
             } else {
                 # convert " to the groff's double quote glyph; it will be
                 # converted back to " in pre_trans. It is needed because
@@ -641,6 +684,8 @@ NEW_LINE:
         my $arguments = $2;
         my @args = splitargs($ref,$arguments);
         if ($macro eq 'B' || $macro eq 'I') {
+            # To keep the space(s), we must introduce some \&
+            @args = map { $_ =~ s/^(\s*)$/\\&$1\\&/s; $_ } @args;
             my $arg=join(" ",@args);
             $arg =~ s/^ +//;
             this_macro_needs_args($macro,$ref,$arg);
@@ -798,8 +843,8 @@ sub pre_trans {
             # Here, we move the punctuation out of the E<...> tag.
             # This is reverted in post_trans.
             # FIXME: To be checked with the French punctuation
-            while ($str =~ m/ +([.,;:\)\]]) *>/s) {
-                $str =~ s/ +([.,;:\)\]]) *>/>$1/s;
+            while ($str =~ m/(?<!\\) +([.,;:\)\]]) *>/s) {
+                $str =~ s/(?<!\\) +([.,;:\)\]]) *>/>$1/s;
             }
         }
         if (defined $t3 and length $t3) {
@@ -876,13 +921,15 @@ sub post_trans {
             if (   ($begin =~ m/(?<!\\)(\\\\)*\\s$/s)
                 or ($begin =~ m/(?<!\\)(\\\\)*\\\((.|E<[gl]t>)$/s)
                 or ($tmp2 =~ m/(?<!\\)(\\\\)*\\[hCv]'([^']|(?<!\\)(\\\\)*\\')*$/)
-                or ($tmp2 =~ m/(?<!\\)(\\\\)*\\\[([^\]]|(?<!\\)(\\\\)*\\\[)*$/)) {
+                or ($tmp2 =~ m/(?<!\\)(\\\\)*\\\[([^\]]|(?<!\\)(\\\\)*\\\[)*$/)
+                or ($tmp2 =~ m/(?<!\\)(\\\\)*\\\*\(.?$/)) {
                 # Do not change - to \- for
                 #  * \s-n (reduce font size)
                 #  * \(.- (a character named '.-', e.g. '<-')
                 #  * inside a \h'...'
                 #  * inside a \C'...'
                 #  * inside a \[...]
+                #  * inside a \*(..
                 #  * inside a \v'...'
                 $tmp = $tmp2."-";
             } else {
@@ -894,7 +941,7 @@ sub post_trans {
     }
 
     # There must not be an end of line inside an inline macro
-    $str =~ s/(E<\.[^>]*)\n([^>]*>)/$1 $2/gs;
+    $str =~ s/(E<\.[^>]*)\n([^>]*>)/$1$2/gs;
 
     # No . or ' on first char, or nroff will think it's a macro
     # * at the beginning of a paragraph, add \& (zero width space) at
@@ -921,8 +968,10 @@ sub post_trans {
 
     # Change ascii non-breaking space to groff one
     my $nbs_out = "\xA0";
-    my $enc_length = Encode::from_to($nbs_out, "latin1",
-                                               $self->get_out_charset);
+    my $enc_length;
+    eval("\$enc_length = Encode::from_to(\$nbs_out, \"latin1\",
+                                         \$self->get_out_charset,
+                                         1)");
     $str =~ s/\Q$nbs_out/\\ /sg if defined $enc_length;
     # No nbsp (said "\ " in groff on the last pos of the line, or groff adds
     # an extra space
@@ -985,20 +1034,63 @@ sub post_trans {
     # $str =~ s/^\\f([BI])(.*?)\\f[RP]$/\.$1 $2/mg;
     
     unless ($mdoc_mode) {
-    $str =~ s/``/\\\*\(lq/sg;
-    $str =~ s/''/\\\*\(rq/sg;
+        my $tmp = "";
+        while ($str =~ m/^(.*?)(``|'')(.*)$/s) {
+            $tmp .= $1;
+            my $q = $2;
+            $str = $3;
+            # There are probably many more exceptions, here are those I could
+            # detect in my manpages.
+            # \*(.'  \*(.`
+            # \*'    \*`
+            # \N'xxx'
+            if ($tmp =~ m/(?<!\\)(?:\\\\)*\\\*\($/s) {
+                $tmp .= $q;
+            } elsif (   $tmp =~ m/(?<!\\)(?:\\\\)*\\\*\(.$/s
+                     or $tmp =~ m/(?<!\\)(?:\\\\)*\\\*$/s
+                     or ($tmp =~ m/(?<!\\)(?:\\\\)*\\N'[0-9]*$/s
+                         and $q eq "''")) {
+                $q =~ m/(.)(.)/;
+                $tmp .= $1;
+                $str = $2.$str;
+            } else {
+                $q =~ s/``/\\\*\(lq/;
+                $q =~ s/''/\\\*\(rq/;
+                $tmp .= $q;
+            }
+        }
+        $str = $tmp.$str;
+    }
+    if (not defined $self->{type}) {
+        $str =~ s/ $//mg;
     }
 
     print STDERR "$str\n" if ($debug{'postrans'});
     return $str;
 }
 sub translate {
-    my ($self,$str,$ref,$type) = @_;
+    my ($self,$str,$ref,$type) = (shift, shift, shift,shift);
     my (%options)=@_;
     my $origstr=$str;
     
     return $str unless (defined $str) && length($str);
     return $str if ($str eq "\n");
+    # Do not translate the strings that only consist of fonts, spaces and
+    # \&. This is useful because we introduced \& in shiftline.
+    if ($str =~ m/^($FONT_RE|\s|\\&)*$/s) {
+        do_fonts($str, $ref);
+        return $str;
+    }
+
+    # If a string is quoted, only translate the argument between the
+    # quotes.
+    if ($options{'wrap'} or $str !~ m/\n/s) {
+        if ($str =~ m/^\"(.*)\"$/s and $1 !~ m/(?<!\\)\"/) {
+            $str = '"'.$self->translate($1, $ref, $type, %options).'"';
+            $str =~ s/\n"$/"\n/s;
+            return $str;
+        }
+    }
 
     $str=pre_trans($self,$str,$ref||$self->{ref},$type);
     $options{'comment'} .= join('\n', @comments);
@@ -1061,8 +1153,9 @@ sub parse{
     if (defined $self->{TT}{'file_in_charset'} and
         length $self->{TT}{'file_in_charset'})
     {
-        $enc_length = Encode::from_to($nbs, "latin-1",
-                                           $self->{TT}{'file_in_charset'});
+        eval ("\$enc_length = Encode::from_to(\$nbs, \"latin-1\",
+                                              \$self->{TT}{'file_in_charset'},
+                                              1)");
     }
     # fall back solution
     $nbs = "PO4A:VERY_IMPROBABLE_STRING_USEDFOR_NON-BREAKING-SPACES"
@@ -1150,7 +1243,7 @@ sub parse{
 	    } else {
 		$self->pushline($line."\n");
 		die wrap_ref_mod($ref, "po4a::man", dgettext("po4a",
-		    "Unknown macro '%s'. Remove it from the document, or provide a patch to the po4a team."), $line);
+		    "Unknown macro '%s'. Remove it from the document, or refer to the Locale::Po4a::Man manpage to see how po4a can handle new macros."), $line);
 	    }
 
 	} elsif ($line =~ /^ +[^. ]/) {
@@ -1211,23 +1304,19 @@ sub parse{
     for my $c (@next_comments) {
 	$self->pushline(".\\\"$c\n");
     }
-} # end of main
 
-# Clear all the memories.
-# This should be called between all documents.
-sub re_init {
-    @comments = ();
+    # reinitialize the module
     @next_comments = ();
     set_regular("R");
     set_font("R");
     set_font("R");
     $mdoc_mode = 0;
-}
+} # end of main
+
 # We can't push the header in the first line of the document, as in the
 # other module, because the first line may contain indications on how the
 # man page must be processed.
 sub docheader {
-    re_init();
     return "";
 }
 
@@ -1263,6 +1352,7 @@ sub splitargs {
     # anymore. Let's play safe.
     $arguments =~ s/\\ /$nbs/g;
     $arguments =~ s/^ +//;
+    $arguments =~ s/\\&"/\\(dq/g;
     foreach my $elem (split (/ +/,$arguments)) {
         print STDERR ">>Seen $elem(buffer=$buffer;esc=$escaped)\n"
             if ($debug{'splitargs'});
@@ -1425,6 +1515,16 @@ sub splitargs {
         my $last_font=$regular_font;
 
         foreach my $elem (@array1) {
+            # Do not touch the fonts in the inline macros
+            # These inline macros may have their argument in bold or italic,
+            # we can't know.
+            if ($str =~ m/E<\.([^>]|E<gt>|E<lt>)*$/s) {
+                # We can't use \\f here, otherwise the font simplifier regexp
+                # will use the fonts of the inline macros.
+                $str .= "PO4A-FAKE-FONT".$elem;
+                next;
+            }
+
             # Replace \fP by the exact font (because some font modifiers will
             # be removed or added, which will break groff's font stack)
             $elem =~ s/^(P|\[\]|\[P\])/$previous_font/s;
@@ -1460,7 +1560,7 @@ sub splitargs {
                                  "po4a::man",
                                  dgettext("po4a",
                                           "Unsupported font in: '%s'."),
-                                 $elem);
+                                 "\\f".$elem);
             }
         }
         # Do some simplification (they don't change the font stack)
@@ -1527,6 +1627,7 @@ sub splitargs {
             }
         }
         $str =~ s/\(CW</CW</sg;
+        $str =~ s/PO4A-FAKE-FONT/\\f/sg;
 
         print STDERR "'$str'\n" if ($debug{'fonts'});
         return $str;
@@ -1587,7 +1688,6 @@ $macro{'TH'}= sub {
 #    print STDERR "TH=$th;titre=$title;sec=$section;date=$date;source=$source;manual=$manual\n";
 
     # Reset the memories
-    re_init();
     $self->push_docheader();
 
     $self->pushmacro($th,
@@ -1847,8 +1947,8 @@ $macro{'hy'}=$macro{'hym'}=$macro{'hys'}=\&untranslated;
 # .ie cond anything  If cond then anything else goto .el.
 # .if cond anything  If cond then anything; otherwise do nothing.
 $macro{'ie'}=$macro{'if'}=sub {
+    my $self = shift;
     if ($groff_code ne "fail") {
-        my $self = shift;
         my $m = $_[0];
         my $paragraph = "@_";
         my ($line,$ref);
@@ -1863,10 +1963,18 @@ $macro{'ie'}=$macro{'if'}=sub {
             $count -= 1 if ($line =~ m/(?<!\\)\\\}/s);
         }
         if ($m eq '.ie') {
+            # The .el line may be preceded by comments
             ($line,$ref)=$self->SUPER::shiftline();
             chomp $line;
+            while ($line =~ m/^[.']\\"/) {
+                $paragraph .= "\n".$line;
+                ($line,$ref)=$self->SUPER::shiftline();
+                chomp $line;
+            }
+
             if ($line !~ m/^[.'][ \t]*el(\s|\\\{)/) {
-                die ".ie without .el\n"
+                die wrap_ref_mod($self->{ref}, "po4a::man", dgettext("po4a",
+                            "The .ie macro must be followed by a .el macro."));
             }
             my $paragraph2 = $line;
             $count = 0;
@@ -1891,8 +1999,8 @@ $macro{'ie'}=$macro{'if'}=sub {
                                               "wrap" => 0) );
         }
     } else {
-        die wrap_mod("po4a::man", dgettext("po4a",
-            "This page uses conditionals with '%s'. Since po4a is not a real groff parser, this is not supported."), $_[1]);
+        die wrap_ref_mod($self->{ref}, "po4a::man", dgettext("po4a",
+            "This page uses conditionals with '%s'. Since po4a is not a real groff parser, this is not supported."), $_[0]);
     }
 };
 # .in  N    Change indent according to N (default scaling indicator m).
@@ -2018,6 +2126,7 @@ $macro{'DS'}=$macro{'LD'}=$macro{'DE'}=\&untranslated;
 $macro{'ID'}=$macro{'BD'}=$macro{'CD'}=\&untranslated;
 $macro{'RD'}=$macro{'KS'}=$macro{'KE'}=\&untranslated;
 $macro{'KF'}=$macro{'B1'}=$macro{'B2'}=\&untranslated;
+$macro{'DA'}=\&translate_joined;
 
 # .pc c  Change page number character
 $macro{'pc'}=\&translate_joined;
@@ -2100,7 +2209,21 @@ $macro{'hw'}=\&translate_each;
 #        (add a flag in Dd, and always check that this flag is set in the
 #        other mdoc macros)
 sub translate_mdoc {
-    my ($self,$macroname,$macroarg)=(shift,shift,join(" ",@_));
+    my ($self,$macroname)=(shift,shift);
+    my $macroarg = "";
+    foreach (@_) {
+        $macroarg.=" " if (length $macroarg);
+        if ($_ =~ m/((?<!\\) |\t)/) {
+            $macroarg.="\"$_\"";
+        } else {
+            $macroarg.=$_;
+        }
+    }
+
+    $self->pushline("$macroname ".$self->t($macroarg)."\n");
+}
+sub translate_mdoc_no_quotes {
+    my ($self,$macroname, $macroarg)=(shift,shift, join(" ", @_));
 
     $self->pushline("$macroname ".$self->t($macroarg)."\n");
 }
@@ -2111,7 +2234,6 @@ sub translate_mdoc {
 $macro{'Dd'}=sub {
     my ($self,$macroname,$macroarg)=(shift,shift,join(" ",@_));
 
-    re_init();
     $mdoc_mode = 1;
     $self->push_docheader();
 
@@ -2129,7 +2251,7 @@ $macro{'Dd'}=sub {
     # Use the mdoc macros
     define_mdoc_macros();
 
-    $self->translate_mdoc($macroname,$macroarg);
+    $self->translate_mdoc_no_quotes($macroname,$macroarg);
 };
 
 sub define_mdoc_macros {
@@ -2145,7 +2267,23 @@ sub define_mdoc_macros {
     # .Sh   Section Headers.
     # (man mdoc indicates only a limited set of valid headers,
     # but it should be OK to translate the header)
-    $macro{'Sh'}=\&translate_mdoc;
+    $macro{'Sh'}= sub {
+        my ($self,$macroname)=(shift,shift);
+        my $macroarg = "";
+        foreach (@_) {
+            $macroarg.=" " if (length $macroarg);
+            if ($_ =~ m/((?<!\\) |\t)/) {
+                $macroarg.="\"$_\"";
+            } else {
+                $macroarg.=$_;
+            }
+        }
+        if ($mdoc{$macroarg}) {
+            $self->pushline("$macroname ".$macroarg."\n");
+        } else {
+            $self->pushline("$macroname ".$self->t($macroarg)."\n");
+        }
+    };
     # .Ss   Subsection Headers.
     $macro{'Ss'}=\&translate_mdoc;
     # .Pp   Paragraph Break.  Vertical space (one line).
