@@ -134,6 +134,9 @@ Coma-separated list of environments which should not be re-wrapped.
 Note that there is a difference between verbatim and no_wrap environments.
 There is no command and comments analysis in verbatim blocks.
 
+If this environment was not already registered, po4a will consider that
+this environment does not take any parameters.
+
 =item exclude_include
 
 Colon-separated list of files that should not be included by \input and
@@ -150,6 +153,9 @@ the document being translated.
 
 Coma-separated list of environments which should be taken as verbatim.
 
+If this environment was not already registered, po4a will consider that
+this environment does not take any parameters.
+
 =back
 
 Using these options permits to override the behaviour of the commands defined
@@ -262,6 +268,14 @@ The notion of environment is expended to the type displayed in the PO file.
 This can be used to split on "\\\\" in the first mandatory argument of the
 title command.  In this case, the environment is title{#1}.
 
+=item % po4a: verbatim environment I<env>
+
+Indicate that I<env> is a verbatim environment.
+Comments and commands will be ignored in this environment.
+
+If this environment was not already registered, po4a will consider that
+this environment does not take any parameters.
+
 =back
 
 =cut
@@ -1048,10 +1062,10 @@ sub parse_definition_line {
         } elsif ($line =~ m/^((?:\{_?\}|\[_?\])*)\s*$/) {
             register_generic_command("$command,$1");
         }
-    } elsif ($line =~ /^environment\s+([+]?\w+\*?)\s+(.*)$/) {
+    } elsif ($line =~ /^environment\s+([+]?\w+\*?)(.*)$/) {
         my $env = $1;
         $line = $2;
-        if ($line =~ m/^((?:\{_?\}|\[_?\])*)\s*$/) {
+        if ($line =~ m/^\s*((?:\{_?\}|\[_?\])*)\s*$/) {
             register_generic_environment("$env,$1");
         }
     } elsif ($line =~ /^separator\s+(\w+(?:\[#[0-9]+\]))\s+\"(.*)\"\s*$/) {
@@ -1191,7 +1205,7 @@ addition to the $self object):
 
 =item A variant
 
-=item An array of (type, argument) tupples
+=item An array of (type, argument) tuples
 
 =item The current environment
 
@@ -1567,6 +1581,8 @@ sub register_verbatim_environment {
     $RE_VERBATIM = "\\\\begin\\{(?:".
                    join("|", split(/ /, $verbatim_environments)).
                    ")\\*?\\}";
+    register_generic_environment("$env,")
+        unless (defined $environments{$env});
 }
 
 ####################################
@@ -1611,6 +1627,8 @@ sub initialize {
     if ($options{'no_wrap'}) {
         foreach (split(/,/, $options{'no_wrap'})) {
             $no_wrap_environments .= " $_";
+            register_generic_environment("$_,")
+                unless (defined $environments{$_});
         }
     }
 
