@@ -10,7 +10,7 @@ use warnings;
 
 use subs qw(makespace);
 use vars qw($VERSION @ISA @EXPORT);
-$VERSION="0.26";
+$VERSION="0.29";
 @ISA = ();
 @EXPORT = qw(process translate 
 	     read write readpo writepo
@@ -832,6 +832,10 @@ sub translate {
 	Encode::from_to($string, $in_charset, $self->{TT}{po_in}->get_charset);
     }
 
+    if (defined $options{'wrapcol'} && $options{'wrapcol'} < 0) {
+# FIXME: should be the parameter given with --width
+        $options{'wrapcol'} = 76 + $options{'wrapcol'};
+    }
     my $transstring = $self->{TT}{po_in}->gettext($string,
 					'wrap'      => $options{'wrap'}||0,
 					'wrapcol'   => $options{'wrapcol'});
