@@ -40,10 +40,6 @@ Locale::Po4a::Xml is a module to help the translation of XML documents into
 other [human] languages. It can also be used as a base to build modules for
 XML-based documents.
 
-Please note that this module is still under heavy development, and not
-distributed in official po4a release since we don't feel it to be mature
-enough. If you insist on trying, check the CVS out.
-
 =cut
 
 package Locale::Po4a::Xml;
@@ -74,6 +70,43 @@ sub parse {
 	map {$self->parse_file($_)} @{$self->{DOCPOD}{infile}};
 }
 
+# @save_holders is a stack of references to ('paragraph', 'translation',
+# 'sub_translations') hashes, where:
+# paragraph is a reference to an array (see paragraph in the
+#           treat_content() subroutine) of strings followed by references.
+#           It contains the @paragraph array as it was before the
+#           processing was interrupted by a tag instroducing a
+#           placeholder.
+# translation is the translation of this level up to now
+# sub_translations is a reference to an array of strings containing the
+#                  translations which must replace the placeholders.
+#
+# If @save_holders only has 1 holder, then we are not processing the
+# content of an holder, we are translating the document.
+my @save_holders;
+
+
+# If we are at the bottom of the stack and there is no <placeholder\d+> in
+# the current translation, we can push the translation in the translated
+# document.
+# Otherwise, we keep the translation in the current holder.
+sub pushline {
+	my ($self, $line) = (shift, shift);
+
+	my $holder_ref = pop @save_holders;
+	my %holder = %$holder_ref;
+	my $translation = $holder{'translation'};
+	$translation .= $line;
+	if (   (scalar @save_holders)
+	    or ($translation =~ m/<placeholder\d+>/s)) {
+		$holder{'translation'} = $translation;
+	} else {
+		$self->SUPER::pushline($translation);
+		$holder{'translation'} = '';
+	}
+	push @save_holders, \%holder;
+}
+
 =head1 TRANSLATING WITH PO4A::XML
 
 This module can be used directly to handle generic XML documents.  This will
@@ -180,6 +213,14 @@ sub initialize {
 	my $self = shift;
 	my %options = @_;
 
+	# Initialize the stack of holders
+	my @paragraph = ();
+	my @sub_translations = ();
+	my %holder = ('paragraph' => \@paragraph,
+	              'translation' => "",
+	              'sub_translations' => \@sub_translations);
+	@save_holders = (\%holder);
+
 	$self->{options}{'nostrip'}=0;
 	$self->{options}{'wrap'}=0;
 	$self->{options}{'caseinsensitive'}=0;
@@ -187,6 +228,7 @@ sub initialize {
 	$self->{options}{'tags'}='';
 	$self->{options}{'attributes'}='';
 	$self->{options}{'inline'}='';
+	$self->{options}{'placeholder'}='';
 	$self->{options}{'doctype'}='';
 	$self->{options}{'nodefault'}='';
 
@@ -208,6 +250,7 @@ sub initialize {
 	$self->{attributes}=();
 	#It will maintain the list of the inline tags
 	$self->{inline}=();
+	$self->{placeholder}=();
 	#list of the tags that must not be set in the tags or inline category
 	#by this module or sub-module (unless specified in an option)
 	$self->{nodefault}=();
@@ -294,6 +337,9 @@ sub found_string {
 		}
 	} elsif ($options->{'type'} eq "attribute") {
 		$comment = "Attribute '".$options->{'attribute'}."' of: ".$self->get_path;
+	} elsif ($options->{'type'} eq "CDATA") {
+		$comment = "CDATA";
+		$wrap = 0;
 	} else {
 		die wrap_ref_mod($ref, "po4a::xml", dgettext("po4a", "Internal error: unknown type identifier '%s'."), $options->{'type'});
 	}
@@ -354,6 +400,9 @@ single string.
 ##### Generic XML tag types #####' 
 
 my @tag_types = ( 
+	{	beginning	=> "!--#",
+		end		=> "--",
+		breaking	=> 0},
 	{	beginning	=> "!--",
 		end		=> "--",
 		breaking	=> 0,
@@ -372,6 +421,11 @@ my @tag_types = (
 		breaking	=> 1,
 		f_extract	=> \&tag_extract_doctype,
 		f_translate	=> \&tag_trans_doctype},
+	{	beginning	=> "![CDATA[",
+		end		=> "",
+		breaking	=> 1,
+		f_extract	=> \&CDATA_extract,
+		f_translate	=> \&CDATA_trans},
 	{	beginning	=> "/",
 		end		=> "",
 		f_breaking	=> \&tag_break_close,
@@ -428,11 +482,21 @@ sub tag_trans_procins {
 sub tag_extract_doctype {
 #TODO
 	my ($self,$remove)=(shift,shift);
-	my ($eof,@tag)=$self->get_string_until(']>',{include=>1,unquoted=>1});
-	if ($eof) {
+
+	# Check if there is an internal subset (between []).
+	my ($eof,@tag)=$self->get_string_until('>',{include=>1,unquoted=>1});
+	my $parity = 0;
+	my $paragraph = "";
+	map { $parity = 1 - $parity; $paragraph.= $parity?$_:""; } @tag;
+	my $found = 0;
+	if ($paragraph =~ m/<.*\[.*</s) {
+		$found = 1
+	}
+
+	if (not $found) {
 		($eof,@tag)=$self->get_string_until('>',{include=>1,remove=>$remove,unquoted=>1});
 	} else {
-		($eof,@tag)=$self->get_string_until(']>',{include=>1,remove=>$remove,unquoted=>1});
+		($eof,@tag)=$self->get_string_until(']\s*>',{include=>1,remove=>$remove,unquoted=>1,regex=>1});
 	}
 	return ($eof,@tag);
 }
@@ -514,6 +578,20 @@ sub tag_trans_close {
 	return $self->join_lines(@tag);
 }
 
+sub CDATA_extract {
+	my ($self,$remove)=(shift,shift);
+        my ($eof, @tag) = $self->get_string_until(']]>',{include=>1,unquoted=>1,remove=>$remove});
+
+	return ($eof, @tag);
+}
+
+sub CDATA_trans {
+	my ($self,@tag)=@_;
+	return $self->found_string($self->join_lines(@tag),
+	                           $tag[1],
+	                           {'type' => "CDATA"});
+}
+
 sub tag_break_alone {
 	my ($self,@tag)=@_;
 	if ($self->tag_in_list($self->get_path."<".
@@ -892,13 +970,86 @@ sub treat_content {
 			# the tag path
 			if ($tag_types[$type]->{'end'} eq "") {
 				if ($tag_types[$type]->{'beginning'} eq "") {
+					# Opening inline tag
+					my $placeholder_regex = join("|", @{$self->{placeholder}});
+					if (length($placeholder_regex) and
+					    $self->get_tag_name(@tag) =~ m/($placeholder_regex)/) { # FIXME
+						# We enter a new holder.
+						# Append a <placeholder#> tag to the current
+						# paragraph, and save the @paragraph in the
+						# current holder.
+						my $holder_ref = pop @save_holders;
+						my %old_holder = %$holder_ref;
+						my $sub_translations_ref = $old_holder{'sub_translations'};
+						my @sub_translations = @$sub_translations_ref;
+
+						push @paragraph, ("<placeholder".($#sub_translations+1).">", $text[1]);
+						my @saved_paragraph = @paragraph;
+
+						$old_holder{'paragraph'} = \@saved_paragraph;
+						push @save_holders, \%old_holder;
+
+						# Then we must push a new holder
+						my @new_paragraph = ();
+						@sub_translations = ();
+						my %new_holder = ('paragraph' => \@new_paragraph,
+						                  'translation' => "",
+						                  'sub_translations' => \@sub_translations);
+						push @save_holders, \%new_holder;
+
+						# The current @paragraph
+						# (for the current holder)
+						# is empty.
+						@paragraph = ();
+					}
 					push @path, $self->get_tag_name(@tag);
 				} elsif ($tag_types[$type]->{'beginning'} eq "/") {
+					# Closing inline tag
+
+					# Check if this is closing the
+					# last opening tag we detected.
 					my $test = pop @path;
 					if (!defined($test) ||
 					    $test ne $tag[0] ) {
 						die wrap_ref_mod($tag[1], "po4a::xml", dgettext("po4a", "Unexpected closing tag </%s> found. The main document may be wrong."), $tag[0]);
 					}
+
+					my $placeholder_regex = join("|", @{$self->{placeholder}});
+					if (length($placeholder_regex) and
+					    $self->get_tag_name(@tag) =~ m/($placeholder_regex)/) {
+						# This closes the current holder.
+
+						# We keep the closing tag in the holder paragraph.
+						push @paragraph, @text;
+						@text = ();
+
+						# Now translate this paragraph if needed.
+						# This will call pushline and append the
+						# translation to the current holder's translation.
+						$self->translate_paragraph($translate, @paragraph);
+
+						# Now that this holder is closed, we can remove
+						# the holder from the stack.
+						my $holder_ref = pop @save_holders;
+						# We need to keep the translation of this holder
+						my %holder = %$holder_ref;
+						my $translation = $holder{'translation'};
+						# Then we store the translation in the previous
+						# holder's sub_translations array
+						my $old_holder_ref = pop @save_holders;
+						my %old_holder = %$old_holder_ref;
+						my $sub_translations_ref = $old_holder{'sub_translations'};
+						my @sub_translations = @$sub_translations_ref;
+						push @sub_translations, $translation;
+						# We also need to restore the @paragraph array, as
+						# it was before we encountered the holder.
+						my $paragraph_ref = $old_holder{'paragraph'};
+						@paragraph = @$paragraph_ref;
+
+						# restore the holder in the stack
+						$old_holder{'sub_translations'} = \@sub_translations;
+						push @save_holders, \%old_holder;
+					}
 				}
 			}
 			push @paragraph, @text;
@@ -933,7 +1084,7 @@ sub treat_content {
 			my ($tmpeof, @tag) = $self->extract_tag($type,0);
 			if ($self->get_tag_name(@tag) eq $path[$#path]) {
 				# The next tag closes the last inline tag.
-				# We nned to temporarily remove the tag from
+				# We need to temporarily remove the tag from
 				# the path before calling breaking_tag
 				my $t = pop @path;
 				if (!$tmpeof and !$self->breaking_tag) {
@@ -989,6 +1140,67 @@ sub treat_content {
 	}
 
 	# Translate the string when needed
+	# This will either push the translation in the translated document or
+	# in the current holder translation.
+	$self->translate_paragraph($translate, @paragraph);
+
+	# Now the paragraph is fully translated.
+	# If we have all the holders' translation, we can replace the
+	# placeholders by their translations.
+	# We must wait to have all the translations because the holders are
+	# numbered.
+	if (scalar @save_holders) {
+		my $holder_ref = pop @save_holders;
+		my %holder = %$holder_ref;
+		my $sub_translations_ref = $holder{'sub_translations'};
+		my $translation = $holder{'translation'};
+		my @sub_translations = @$sub_translations_ref;
+
+		# Count the number of <placeholder\d+> in $translation
+		my $count = 0;
+		my $str = $translation;
+		while ($str =~ m/^.*?<placeholder\d+>(.*)$/s) {
+			$count += 1;
+			$str = $1;
+		}
+
+		if (scalar(@sub_translations) == $count) {
+			# OK, all the holders of the current paragraph are
+			# closed (and translated).
+			# Replace them by their translation.
+			while ($translation =~ m/^(.*?)<placeholder(\d+)>(.*)$/s) {
+				# FIXME: we could also check that
+				#          * the holder exists
+				#          * all the holders are used
+				$translation = $1.$sub_translations[$2].$3;
+			}
+			# We have our translation
+			$holder{'translation'} = $translation;
+			# And there is no need for any holder in it.
+			@sub_translations = ();
+			$holder{'sub_translations'} = \@sub_translations;
+# FIXME: is it alright if a document ends by a placeholder?
+		}
+		# Either we don't have all the holders, either we have the
+		# final translation.
+		# We must keep the current holder at the top of the stack.
+		push @save_holders, \%holder;
+	}
+
+	# Push the trailing blanks
+	if ($blank ne "") {
+		$self->pushline($blank);
+	}
+	return $eof;
+}
+
+# Translate a @paragraph array of (string, reference).
+# The $translate argument indicates if the strings must be translated or
+# just pushed
+sub translate_paragraph {
+	my ($self, $translate) = (shift, shift);
+	my @paragraph = @_;
+
 	if ( length($self->join_lines(@paragraph)) > 0 ) {
 		my $struc = $self->get_path;
 		my $options = $self->tag_in_list($struc,@{$self->{tags}});
@@ -1008,18 +1220,10 @@ sub treat_content {
 			$self->pushline($self->recode_skipped_text($self->join_lines(@paragraph)));
 		}
 	}
-
-	# Push the trailing blanks
-	if ($blank ne "") {
-		$self->pushline($blank);
-	}
-	return $eof;
 }
 
 
 
-
-
 =head2 WORKING WITH THE MODULE OPTIONS
 
 =over 4
@@ -1048,6 +1252,10 @@ sub treat_options {
 	$self->{options}{'inline'} =~ /\s*(.*)\s*/s;
 	my @list_inline = split(/\s+/s,$1);
 	$self->{inline} = \@list_inline;
+
+	$self->{options}{'placeholder'} =~ /\s*(.*)\s*/s;
+	my @list_placeholder = split(/\s+/s,$1);
+	$self->{placeholder} = \@list_placeholder;
         
 	$self->{options}{'nodefault'} =~ /\s*(.*)\s*/s;
 	my %list_nodefault;
@@ -1090,25 +1298,27 @@ This ensures that the searched text is outside any quotes
 sub get_string_until {
 	my ($self,$search) = (shift,shift);
 	my $options = shift;
-	my ($include,$remove,$unquoted) = (0,0,0);
+	my ($include,$remove,$unquoted, $regex) = (0,0,0,0);
 
 	if (defined($options->{include})) { $include = $options->{include}; }
 	if (defined($options->{remove})) { $remove = $options->{remove}; }
 	if (defined($options->{unquoted})) { $unquoted = $options->{unquoted}; }
+	if (defined($options->{regex})) { $regex = $options->{regex}; }
 
 	my ($line,$ref) = $self->shiftline();
 	my (@text,$paragraph);
 	my ($eof,$found) = (0,0);
 
+	$search = "\Q$search\E" unless $regex;
 	while (defined($line) and !$found) {
 		push @text, ($line,$ref);
 		$paragraph .= $line;
 		if ($unquoted) {
-			if ( $paragraph =~ /^((\".*?\")|(\'.*?\')|[^\"\'])*\Q$search\E.*/s ) {
+			if ( $paragraph =~ /^((\".*?\")|(\'.*?\')|[^\"\'])*$search.*/s ) {
 				$found = 1;
 			}
 		} else {
-			if ( $paragraph =~ /.*\Q$search\E.*/s ) {
+			if ( $paragraph =~ /.*$search.*/s ) {
 				$found = 1;
 			}
 		}
@@ -1122,16 +1332,16 @@ sub get_string_until {
 	if ( $found ) {
 		$line = "";
 		if($unquoted) {
-			$text[$#text-1] =~ /^(((\".*?\")|(\'.*?\')|[^\"\'])*?\Q$search\E)(.*)/s;
-			$text[$#text-1] = $1;
-			$line = $5;
+			$paragraph =~ /^(?:(?:\".*?\")|(?:\'.*?\')|[^\"\'])*?$search(.*)$/s;
+			$line = $1;
+			$text[$#text-1] =~ s/\Q$line\E$//s;
 		} else {
-			$text[$#text-1] =~ /(.*?\Q$search\E)(.*)/s;
-			$text[$#text-1] = $1;
-			$line = $2;
+			$paragraph =~ /$search(.*)$/s;
+			$line = $1;
+			$text[$#text-1] =~ s/\Q$line\E$//s;
 		}
 		if(!$include) {
-			$text[$#text-1] =~ /(.*)(\Q$search\E.*)/s;
+			$text[$#text-1] =~ /(.*)($search.*)/s;
 			$text[$#text-1] = $1;
 			$line = $2.$line;
 		}
